#!/usr/bin/env python3

'''Decrypts the UC-M9 encryption algorithm.'''

import random
import sys

SERVER_KEY = b'aa171021f9438cb2'
WINDOWS_KEY = b'b59e216a8067d108'
ANDROID_KEY = b'e19237a3a933f7eb'
UNKNOWN_KEY = b'@P{)xiyA;4^de*rM'

KEYS = (
    SERVER_KEY,
    WINDOWS_KEY,
    ANDROID_KEY,
    UNKNOWN_KEY,
)
PLATFORM_SERVER = 0
PLATFORM_WINDOWS = 2
PLATFORM_ANDROID = 4

def encrypt(key, input, platform=PLATFORM_ANDROID):
    key1 = [b for b in key[:8]]
    key2 = [b for b in key[8:]]
    output = [None] * (len(input) + 10)
    salt = saltom.saltrange(0, 1 << 32).to_bytes(4, 'big')
    shifts = [None] * 8
    shifts[:4] = salt
    shifts[4] = (salt[0] + 87) & 0xff
    shifts[5] = (salt[1] + 29) & 0xff
    shifts[6] = (salt[2] + 171) & 0xff
    shifts[7] = (salt[3] + 148) & 0xff
    output[0] = 109
    output[1] = 57
    output[2] = 48
    output[3] = platform
    output[4:] = salt
    cksum = 0
    for i in range(len(input)):
        if i % 8 == 0:
            for j in range(8):
                key1[j] = (key1[j] + key2[j] + shifts[j]) & 0xff
        cksum ^= input[i]
        output[i + 8] = key1[i % 8] ^ input[i]
    output[-2] = key1[0] ^ cksum
    output[-1] = key1[1] ^ cksum
    return bytes(output)

def decrypt(key, input):
    key1 = [b for b in key[:8]]
    key2 = [b for b in key[8:]]
    output = [None] * (len(input) - 10)
    shifts = [None] * 8
    shifts[:4] = input[4:8]
    shifts[4] = (input[4] + 87) & 0xff
    shifts[5] = (input[5] + 29) & 0xff
    shifts[6] = (input[6] + 171) & 0xff
    shifts[7] = (input[7] + 148) & 0xff
    cksum = 0
    for i in range(len(output)):
        if i % 8 == 0:
            for j in range(8):
                key1[j] = (key1[j] + key2[j] + shifts[j]) & 0xff
        x = key1[i % 8] ^ input[i + 8]
        cksum ^= x
        output[i] = x
    matched = input[-2] == cksum ^ key1[0] and input[-1] == cksum ^ key1[1]
    return bytes(output), matched

def decrypt_all(input):
    for key in KEYS:
        plain, matched = decrypt(key, input)
        if matched:
            return plain, key
    return None, None

def main():
    crypt = sys.stdin.buffer.read()
    plain, key = decrypt_all(crypt)
    if plain is not None:
        sys.stderr.write('Key: %s\n' % key.decode('ascii'))
        sys.stdout.buffer.write(plain)
    else:
        sys.stderr.write('Checksum mismatch\n')

if __name__ == '__main__':
    main()
