#!/usr/bin/env python3

'''Parse protobuf and produce human-readable output.  This is like 'protoc
--decode_raw' except it is more verbose in that it prints alternative
interpretations of binary data (both unsigned and signed and, when applicable,
a floating point representation).
'''

import struct
import sys

from parse import parse_file

FLOAT = struct.Struct('f')
DOUBLE = struct.Struct('d')

def signed(n):
    v = n >> 1
    if n & 1:
        v = ~v
    return v

def float(n):
    return FLOAT.unpack(n.to_bytes(4, 'big'))[0]

def double(n):
    return DOUBLE.unpack(n.to_bytes(8, 'big'))[0]

def print_raw(fields, indent=0):
    for field_number, wire_type, v in fields:
        if isinstance(v, list):
            print(' ' * indent + '%d {' % field_number)
            print_raw(v, indent + 2)
            print(' ' * indent + '}')
        else:
            if wire_type == 0:
                if v != 0:
                    v = ' | '.join(repr(x) for x in (v, signed(v)))
            elif wire_type == 1:
                if v != 0:
                    v = ' | '.join(repr(x) for x in (v, signed(v), double(v)))
            elif wire_type == 2:
                v = repr(v)[1:]
            elif wire_type == 5:
                if v != 0:
                    v = ' | '.join(repr(x) for x in (v, signed(v), float(v)))
            else:
                v = repr(v)
            print(' ' * indent + '%d: %s' % (field_number, v))

def main():
    print_raw(parse_file(sys.stdin.buffer))

if __name__ == '__main__':
    main()
