#!/usr/bin/env python3

'''Decrypts the UC-XOR algorithm.'''

import sys

KEY = b'\xee\xb9\xe9\xb3\x81\x8e\x97\xa7'

def decrypt(crypt):
    n = len(KEY)
    plain_bytes = []
    cksum = 0
    for i, b in enumerate(crypt[:-2]):
        x = b ^ KEY[i % n]
        cksum ^= x
        plain_bytes.append(x)
    plain = bytes(plain_bytes)
    matched = crypt[-2] == cksum ^ KEY[0] and crypt[-1] == cksum ^ KEY[1]
    return plain, matched

def main():
    crypt = sys.stdin.buffer.read()
    plain, matched = decrypt(crypt)
    if not matched:
        sys.stderr.write('Checksum mismatch\n')
        sys.stdout.buffer.write(plain)
    else:
        sys.stdout.buffer.write(plain)

if __name__ == '__main__':
    main()
